package com.scbfkj.uni.service;

import com.scbfkj.uni.library.DataFormatUtil;
import com.scbfkj.uni.library.UniReturnUtil;
import com.scbfkj.uni.process.DataBase;
import com.scbfkj.uni.system.Config;
import com.scbfkj.uni.system.ScheduleTask;
import com.scbfkj.uni.system.ScheduleUtil;
import org.springframework.util.StringUtils;

import java.time.LocalDateTime;
import java.util.*;

public class ControlService {


    public static Map<String, ScheduleTask> ScheduleTaskMaps = new HashMap<>();
    /**
     * 服务启动
     *
     * @param serviceId
     * @return
     */
    public static Map<String,Object> start(String serviceId) throws Exception {
        //采集数据源及参数配置
        List<Map<String, Object>> collectServiceList = DataBase.query(Config.centerConnectionStr,"SELECT SI.* FROM  serviceinfo SI WHERE SI.serviceType ='4'  AND serviceID = ?", Collections.singletonList(new Object[]{serviceId}));

        if (Objects.isNull(collectServiceList) || collectServiceList.isEmpty()) {
            return UniReturnUtil.fail("采集服务不存在");
        }
        Map<String, Object> collectServiceMaps = collectServiceList.get(0); //采集服务
        Integer loopCount = Objects.isNull (collectServiceMaps.get("loopCount")) ? null : Integer.parseInt(collectServiceMaps.get("loopCount").toString());
        int frequencyCount = Objects.isNull (collectServiceMaps.get("frequencyCount")) ? 1 : Integer.parseInt(collectServiceMaps.get("frequencyCount").toString());
        int frequencyUnit = Objects.isNull (collectServiceMaps.get("frequencyUnit")) ? 1 : Integer.parseInt(collectServiceMaps.get("frequencyUnit").toString());
        int taskType = Objects.isNull (collectServiceMaps.get("taskType")) ? 1 : Integer.parseInt(collectServiceMaps.get("taskType").toString());
        Object cronExpress = collectServiceMaps.get("cronExpress");
        LocalDateTime taskValidDate = DataFormatUtil.toDateTime(collectServiceMaps.get("taskValid"));
        LocalDateTime taskInvalidDate = DataFormatUtil.toDateTime(collectServiceMaps.get("taskInvalid"));
        //创建定时任务：
        ScheduleTask scheduleTask = ScheduleTaskMaps.get(serviceId);
        if (Objects.isNull(scheduleTask)) {
            scheduleTask = new ScheduleTask(serviceId, loopCount, taskValidDate, taskInvalidDate);
            ScheduleTaskMaps.put(serviceId, scheduleTask);
        }
        // todo 记录日志
        boolean start = ScheduleUtil.start(scheduleTask, (frequencyCount * frequencyUnit), cronExpress, taskType);
//        LogUtils.log("start: 2", "0", null, (start ? "启动成功" : "启动失败"), serviceId,  null, null, null,null);//无条件记录数据接收日志
        if (start) {
            return UniReturnUtil.success("启动成功");
        }
        DataBase.updateBatch(Config.centerConnectionStr,"update serviceinfo set runState = 1 where  serviceid =?",Collections.singletonList(new Object[]{serviceId}));
        ScheduleTaskMaps.remove(serviceId);
        return UniReturnUtil.fail("启动失败");
    }

    /**
     * 服务停止
     *
     * @param service_id
     * @return
     */
    public static Map<String,Object> stop(String service_id) throws Exception {
        ScheduleTask scheduleTask = ScheduleTaskMaps.get(service_id);
        if (Objects.isNull(scheduleTask)) {
            return UniReturnUtil.fail("服务： " + service_id + " 已经停止");
        }
        boolean reset = ScheduleUtil.cancel(scheduleTask);
        ScheduleTaskMaps.remove(service_id);
        DataBase.updateBatch(Config.centerConnectionStr,"update serviceinfo set runState = 0 where  serviceID =?",Collections.singletonList(new Object[]{service_id}));
//        LogUtils.log("stop: 2", "0", null, (reset ? "停止成功" : "停止失败"), service_id, null, null, null,null);//无条件记录数据接收日志
        return UniReturnUtil.success("停止" + (reset ? "成功" : "失败"));
    }

    public static Map<String,Object> startOrStop(Map<String, Object> params, String statue) throws Exception {
        Object serviceId1 = params.get("serviceId");
        String serviceId = Objects.isNull(serviceId1) ? null : serviceId1.toString();
        if (!StringUtils.hasText(serviceId)) {
            return  UniReturnUtil.fail("服务ID不能为空");
        }
        List<Map<String, Object>> serviceTypeData = DataBase.query(Config.centerConnectionStr,"select serviceType from serviceinfo where serviceID=?", Collections.singletonList(new Object[]{serviceId}));

        if (Objects.nonNull(serviceTypeData) && Objects.equals("4",serviceTypeData.get(0).get("serviceType").toString())) {
            if (statue.equals("0")) {
                return stop(serviceId);
            }
            return start(serviceId);
        }
//        ObjectMap.getordropInput(serviceId,false);
        return UniReturnUtil.success("重启成功");
    }
}
