package com.scbfkj.uni.system;

import com.scbfkj.uni.service.DataProcessService;
import org.springframework.scheduling.concurrent.ThreadPoolTaskScheduler;
import org.springframework.scheduling.support.CronTrigger;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ScheduledFuture;

public class ScheduleUtil {


    private static Map<String, DataProcessService> dataProcessMaps = new HashMap<>(); //数据接收

    private static DataProcessService getordropInput(String serviceId, boolean isGet) {
        if (isGet) {
            DataProcessService dataProcess = dataProcessMaps.get(serviceId);
            if (Objects.nonNull(dataProcess) && Objects.nonNull(dataProcess.getErrorMessage())) {
                dataProcess.close();
                dataProcess = null;
                dataProcessMaps.remove(serviceId);
            }
            if (!dataProcessMaps.containsKey(serviceId)) {
                dataProcessMaps.put(serviceId, new DataProcessService(serviceId));
            }
            return dataProcessMaps.get(serviceId);
        } else {
            DataProcessService datacollect = dataProcessMaps.get(serviceId);
            if (Objects.nonNull(datacollect)) {
                datacollect.close();
                datacollect = null;
            }
            dataProcessMaps.remove(serviceId);
            return null;
        }
    }

    public static DataProcessService getordropInput(String serviceId) {
        return getordropInput(serviceId, true);
    }


    private static ThreadPoolTaskScheduler threadPoolTaskScheduler = new ThreadPoolTaskScheduler();

    private static Map<String, ScheduledFuture<?>> scheduledFutureMap = new HashMap<>();

    static {

        threadPoolTaskScheduler.setPoolSize(8);
        threadPoolTaskScheduler.initialize();
        System.out.println("定时任务线程池启动");
    }

    /**
     * 启动
     *
     * @param scheduleTask 定时任务
     */
    public static boolean start(ScheduleTask scheduleTask, int cycleFrequency, Object cronExpress, int taskType) {
        System.out.println("启动定时任务线程 taskId " + scheduleTask.getId());// 设置时间比当前时间晚了
        ScheduledFuture<?> scheduledFuture;
        if (taskType == 1) {
            scheduledFuture = threadPoolTaskScheduler.scheduleWithFixedDelay(scheduleTask, new Date(), cycleFrequency);
        } else {
            if (Objects.isNull(cronExpress)) {
//                todo 记录日志
                return false;
            }
            scheduledFuture = threadPoolTaskScheduler.schedule(scheduleTask, new CronTrigger(cronExpress.toString()));
        }
        scheduledFutureMap.put(scheduleTask.getId(), scheduledFuture);
        return true;
    }

    /**
     * 取消
     *
     * @param scheduleTask 定时任务
     */
    public static boolean cancel(ScheduleTask scheduleTask) {
        System.out.println("关闭定时任务线程 taskId " + scheduleTask.getId());
        String serviceId = scheduleTask.getId();
        ScheduledFuture<?> scheduledFuture = scheduledFutureMap.get(serviceId);
        if (scheduledFuture != null && !scheduledFuture.isCancelled()) {
            scheduledFuture.cancel(false);
        }
        scheduledFutureMap.remove(serviceId);
        //根据服务ID获取所有算法，进行统一关闭
        getordropInput(serviceId,false);
        return true;
    }
}
